package util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Map;

import org.junit.Test;
import org.springframework.cache.Cache;

/**
 * Test cases for {@link LRUMapCacheManager}.
 */
public class LRUMapCacheManagerTest {

    private LRUMapCacheManager cacheManager = new LRUMapCacheManager();

    @Test
    public void loadCachesReturnsEmptyList() {
        assertEquals(0, cacheManager.loadCaches().size());
    }

    @Test
    public void maxCacheSizeIsApplied() {
        int cacheMaxSize = 3;

        cacheManager.setMaxCacheSize(cacheMaxSize);

        Cache cache = cacheManager.getCache("any");

        cache.put("one", Boolean.TRUE);
        cache.put("two", Boolean.TRUE);
        cache.put("three", Boolean.TRUE);
        cache.put("four", Boolean.TRUE);
        cache.put("five", Boolean.TRUE);

        assertEquals(cacheMaxSize, ((Map<?, ?>) cache.getNativeCache()).size());
    }

    @Test
    public void getCacheReturnsCachedIntance() {
        String cacheName = "dummy";

        Cache cache = cacheManager.getCache(cacheName);

        assertTrue(cache == cacheManager.getCache(cacheName));
    }

    @Test
    public void cacheNameMatches() {
        String cacheName = "any";

        assertEquals(cacheName, cacheManager.getCache(cacheName).getName());
    }

    @Test
    public void testCacheLifecycle() {
        String key1 = "one";
        String key2 = "two";

        Cache cache = cacheManager.getCache("any");

        cache.put(key1, Boolean.TRUE);
        cache.put(key2, 12);

        assertEquals(Boolean.TRUE, cache.get(key1).get());
        assertEquals(12, cache.get(key2).get());

        assertEquals(2, ((Map<?, ?>) cache.getNativeCache()).size());

        cache.evict(key1);

        assertEquals(1, ((Map<?, ?>) cache.getNativeCache()).size());
        assertNull(cache.get(key1));
        assertEquals(Integer.valueOf(12), cache.get(key2, () -> 14));

        cache.putIfAbsent(key2, 20);

        assertEquals(12, cache.get(key2).get());

        cache.clear();

        assertEquals(0, ((Map<?, ?>) cache.getNativeCache()).size());
        assertEquals(Integer.valueOf(14), cache.get(key2, () -> 14));
        assertEquals(1, ((Map<?, ?>) cache.getNativeCache()).size());
        assertEquals(14, cache.get(key2).get());

        cache.putIfAbsent(key2, 24);

        assertEquals(14, cache.get(key2).get());

        cache.putIfAbsent(key2, 20);
    }
}
